@@ -1,5 +1,7 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; c-indent-level: 8 -*- */
-/* pdfdocument.h: Implementation of EvDocument for PDF
+/* this file is part of evince, a gnome document viewer
+ *
+ * Copyright (C) 2009, Juanjo Marín <juanj.marin@juntadeandalucia.es>
  * Copyright (C) 2004, Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -48,6 +50,7 @@
 #include "ev-document-layers.h"
 #include "ev-document-print.h"
 #include "ev-document-annotations.h"
+#include "ev-document-attachments.h"
 #include "ev-selection.h"
 #include "ev-transition-effect.h"
 #include "ev-attachment.h"
@@ -62,6 +65,13 @@
 #define HAVE_CAIRO_PRINT
 #endif
 
+/* fields from the XMP Rights Management Schema, XMP Specification Sept 2005, pag. 45 */
+#define LICENSE_MARKED "/x:xmpmeta/rdf:RDF/rdf:Description/xmpRights:Marked"
+#define LICENSE_TEXT "/x:xmpmeta/rdf:RDF/rdf:Description/dc:rights/rdf:Alt/rdf:li[lang('%s')]"
+#define LICENSE_WEB_STATEMENT "/x:xmpmeta/rdf:RDF/rdf:Description/xmpRights:WebStatement"
+/* license field from Creative Commons schema, http://creativecommons.org/ns */
+#define LICENSE_URI "/x:xmpmeta/rdf:RDF/rdf:Description/cc:license/@rdf:resource"
+
 typedef struct {
 	PdfDocument *document;
 	char *text;
@@ -91,12 +101,12 @@ typedef struct {
 
 struct _PdfDocumentClass
 {
-	GObjectClass parent_class;
+	EvDocumentClass parent_class;
 };
 
 struct _PdfDocument
 {
-	GObject parent_instance;
+	EvDocument parent_instance;
 
 	PopplerDocument *document;
 	gchar *password;
@@ -112,7 +122,6 @@ struct _PdfDocument
 	GList *layers;
 };
 
-static void pdf_document_document_iface_init             (EvDocumentIface            *iface);
 static void pdf_document_security_iface_init             (EvDocumentSecurityIface    *iface);
 static void pdf_document_document_thumbnails_iface_init  (EvDocumentThumbnailsIface  *iface);
 static void pdf_document_document_links_iface_init       (EvDocumentLinksIface       *iface);
@@ -124,6 +133,7 @@ static void pdf_document_document_layers_iface_init      (EvDocumentLayersIface
 static void pdf_document_document_print_iface_init       (EvDocumentPrintIface       *iface);
 #endif
 static void pdf_document_document_annotations_iface_init (EvDocumentAnnotationsIface *iface);
+static void pdf_document_document_attachments_iface_init (EvDocumentAttachmentsIface *iface);
 static void pdf_document_find_iface_init                 (EvDocumentFindIface        *iface);
 static void pdf_document_file_exporter_iface_init        (EvFileExporterIface        *iface);
 static void pdf_selection_iface_init                     (EvSelectionIface           *iface);
@@ -134,12 +144,16 @@ static void pdf_document_thumbnails_get_dimensions       (EvDocumentThumbnails
 							  gint                       *height);
 static int  pdf_document_get_n_pages			 (EvDocument                 *document);
 
-static EvLinkDest *ev_link_dest_from_dest   (PdfDocument       *pdf_document,
-					     PopplerDest       *dest);
-static EvLink     *ev_link_from_action      (PdfDocument       *pdf_document,
-					     PopplerAction     *action);
-static void        pdf_document_search_free (PdfDocumentSearch *search);
-static void        pdf_print_context_free   (PdfPrintContext   *ctx);
+static EvLinkDest *ev_link_dest_from_dest    (PdfDocument       *pdf_document,
+					      PopplerDest       *dest);
+static EvLink     *ev_link_from_action       (PdfDocument       *pdf_document,
+					      PopplerAction     *action);
+static void        pdf_document_search_free  (PdfDocumentSearch *search);
+static void        pdf_print_context_free    (PdfPrintContext   *ctx);
+static gboolean    attachment_save_to_buffer (PopplerAttachment *attachment,
+					      gchar            **buffer,
+					      gsize             *buffer_size,
+					      GError           **error);
 
 EV_BACKEND_REGISTER_WITH_CODE (PdfDocument, pdf_document,
 			 {
@@ -163,6 +177,8 @@ EV_BACKEND_REGISTER_WITH_CODE (PdfDocument, pdf_document,
 #endif
 				 EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_ANNOTATIONS,
 								 pdf_document_document_annotations_iface_init);
+				 EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_ATTACHMENTS,
+								 pdf_document_document_attachments_iface_init);
 				 EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_FIND,
 								 pdf_document_find_iface_init);
 				 EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_FILE_EXPORTER,
@@ -229,14 +245,6 @@ pdf_document_dispose (GObject *object)
 	G_OBJECT_CLASS (pdf_document_parent_class)->dispose (object);
 }
 
-static void
-pdf_document_class_init (PdfDocumentClass *klass)
-{
-	GObjectClass *g_object_class = G_OBJECT_CLASS (klass);
-
-	g_object_class->dispose = pdf_document_dispose;
-}
-
 static void
 pdf_document_init (PdfDocument *pdf_document)
 {
@@ -361,122 +369,6 @@ pdf_document_get_page_label (EvDocument *document,
 	return label;
 }
 
-static gboolean
-pdf_document_has_attachments (EvDocument *document)
-{
-	PdfDocument *pdf_document;
-
-	pdf_document = PDF_DOCUMENT (document);
-
-	return poppler_document_has_attachments (pdf_document->document);
-}
-
-struct SaveToBufferData {
-	gchar *buffer;
-	gsize len, max;
-};
-
-static gboolean
-attachment_save_to_buffer_callback (const gchar  *buf,
-				    gsize         count,
-				    gpointer      user_data,
-				    GError      **error)
-{
-	struct SaveToBufferData *sdata = (SaveToBufferData *)user_data;
-	gchar *new_buffer;
-	gsize new_max;
-
-	if (sdata->len + count > sdata->max) {
-		new_max = MAX (sdata->max * 2, sdata->len + count);
-		new_buffer = (gchar *)g_realloc (sdata->buffer, new_max);
-
-		sdata->buffer = new_buffer;
-		sdata->max = new_max;
-	}
-	
-	memcpy (sdata->buffer + sdata->len, buf, count);
-	sdata->len += count;
-	
-	return TRUE;
-}
-
-static gboolean
-attachment_save_to_buffer (PopplerAttachment  *attachment,
-			   gchar             **buffer,
-			   gsize              *buffer_size,
-			   GError            **error)
-{
-	static const gint initial_max = 1024;
-	struct SaveToBufferData sdata;
-
-	*buffer = NULL;
-	*buffer_size = 0;
-
-	sdata.buffer = (gchar *) g_malloc (initial_max);
-	sdata.max = initial_max;
-	sdata.len = 0;
-
-	if (! poppler_attachment_save_to_callback (attachment,
-						   attachment_save_to_buffer_callback,
-						   &sdata,
-						   error)) {
-		g_free (sdata.buffer);
-		return FALSE;
-	}
-
-	*buffer = sdata.buffer;
-	*buffer_size = sdata.len;
-	
-	return TRUE;
-}
-
-static GList *
-pdf_document_get_attachments (EvDocument *document)
-{
-	PdfDocument *pdf_document;
-	GList *attachments;
-	GList *list;
-	GList *retval = NULL;
-
-	pdf_document = PDF_DOCUMENT (document);
-
-	if (!pdf_document_has_attachments (document))
-		return NULL;
-
-	attachments = poppler_document_get_attachments (pdf_document->document);
-	
-	for (list = attachments; list; list = list->next) {
-		PopplerAttachment *attachment;
-		EvAttachment *ev_attachment;
-		gchar *data = NULL;
-		gsize size;
-		GError *error = NULL;
-
-		attachment = (PopplerAttachment *) list->data;
-
-		if (attachment_save_to_buffer (attachment, &data, &size, &error)) {
-			ev_attachment = ev_attachment_new (attachment->name,
-							   attachment->description,
-							   attachment->mtime,
-							   attachment->ctime,
-							   size, data);
-			
-			retval = g_list_prepend (retval, ev_attachment);
-		} else {
-			if (error) {
-				g_warning ("%s", error->message);
-				g_error_free (error);
-
-				g_free (data);
-			}
-		}
-
-		g_object_unref (attachment);
-	}
-
-	return g_list_reverse (retval);
-}
-
 static cairo_surface_t *
 pdf_page_render (PopplerPage     *page,
 		 gint             width,
@@ -487,14 +379,11 @@ pdf_page_render (PopplerPage     *page,
 
 #ifdef HAVE_POPPLER_PAGE_RENDER
 	cairo_t *cr;
-	
-	surface = cairo_image_surface_create (CAIRO_FORMAT_RGB24,
+
+	surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
 					      width, height);
-	memset (cairo_image_surface_get_data (surface), 0xff,
-		cairo_image_surface_get_height (surface) *
-		cairo_image_surface_get_stride (surface));
-	
 	cr = cairo_create (surface);
+
 	switch (rc->rotation) {
 	        case 90:
 			cairo_translate (cr, width, 0);
@@ -511,6 +400,11 @@ pdf_page_render (PopplerPage     *page,
 	cairo_scale (cr, rc->scale, rc->scale);
 	cairo_rotate (cr, rc->rotation * G_PI / 180.0);
 	poppler_page_render (page, cr);
+
+	cairo_set_operator (cr, CAIRO_OPERATOR_DEST_OVER);
+	cairo_set_source_rgb (cr, 1., 1., 1.);
+	cairo_paint (cr);
+
 	cairo_destroy (cr);
 #else /* HAVE_POPPLER_PAGE_RENDER */
 	GdkPixbuf *pixbuf;
@@ -558,51 +452,18 @@ pdf_document_render (EvDocument      *document,
 				width, height, rc);
 }
 
-/* EvDocumentSecurity */
-
-static gboolean
-pdf_document_has_document_security (EvDocumentSecurity *document_security)
-{
-	/* FIXME: do we really need to have this? */
-	return FALSE;
-}
-
-static void
-pdf_document_set_password (EvDocumentSecurity *document_security,
-			   const char         *password)
-{
-	PdfDocument *document = PDF_DOCUMENT (document_security);
-
-	if (document->password)
-		g_free (document->password);
-
-	document->password = g_strdup (password);
-}
-
-
 /* reference:
 http://www.pdfa.org/lib/exe/fetch.php?id=pdfa%3Aen%3Atechdoc&cache=cache&media=pdfa:techdoc:tn0001_pdfa-1_and_namespaces_2008-03-18.pdf */
 static char *
-pdf_document_get_format_from_metadata (const char *metadata)
+pdf_document_get_format_from_metadata (xmlDocPtr          doc,
+				       xmlXPathContextPtr xpathCtx)
 {
-	xmlDocPtr doc;
-	xmlXPathContextPtr xpathCtx;
 	xmlXPathObjectPtr xpathObj;
 	xmlChar *part = NULL;
 	xmlChar *conf = NULL;
 	char *result = NULL;
 	int i;
 
-	doc = xmlParseMemory (metadata, strlen (metadata));
-	if (doc == NULL)
-		return NULL;      /* invalid xml metadata */
-
-	xpathCtx = xmlXPathNewContext (doc);
-	if (xpathCtx == NULL) {
-		xmlFreeDoc (doc);
-		return NULL;      /* invalid xpath context */
-	}
-
 	/* add pdf/a namespaces */
 	xmlXPathRegisterNs (xpathCtx, BAD_CAST "x", BAD_CAST "adobe:ns:meta/");
 	xmlXPathRegisterNs (xpathCtx, BAD_CAST "rdf", BAD_CAST "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
@@ -660,12 +521,159 @@ pdf_document_get_format_from_metadata (const char *metadata)
 	/* Cleanup */
 	xmlFree (part);
 	xmlFree (conf);
-	xmlXPathFreeContext (xpathCtx);
-	xmlFreeDoc (doc);
 
 	return result;
 }
 
+static EvDocumentLicense *
+pdf_document_get_license_from_metadata (xmlDocPtr          doc,
+					xmlXPathContextPtr xpathCtx)
+{
+	xmlXPathObjectPtr xpathObj;
+	xmlChar *marked = NULL;
+	const char *language_string;
+	char  *aux;
+	gchar **tags;
+	gchar *tag, *tag_aux;
+	int i, j;
+	EvDocumentLicense *license;
+
+	/* register namespaces */
+	xmlXPathRegisterNs (xpathCtx, BAD_CAST "x", BAD_CAST "adobe:ns:meta/");
+	xmlXPathRegisterNs (xpathCtx, BAD_CAST "rdf", BAD_CAST "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
+	xmlXPathRegisterNs (xpathCtx, BAD_CAST "dc", BAD_CAST "http://purl.org/dc/elements/1.1/");
+	/* XMP Rights Management Schema */
+	xmlXPathRegisterNs (xpathCtx, BAD_CAST "xmpRights", BAD_CAST "http://ns.adobe.com/xap/1.0/rights/");
+	/* Creative Commons Schema */
+	xmlXPathRegisterNs (xpathCtx, BAD_CAST "cc", BAD_CAST "http://creativecommons.org/ns#");
+
+	/* checking if the document has been marked as defined on the XMP Rights
+	 * Management Schema */
+	xpathObj = xmlXPathEvalExpression (BAD_CAST LICENSE_MARKED, xpathCtx);
+	if (xpathObj != NULL) {
+		if (xpathObj->nodesetval != NULL &&
+		    xpathObj->nodesetval->nodeNr != 0)
+			marked = xmlNodeGetContent (xpathObj->nodesetval->nodeTab[0]);
+		xmlXPathFreeObject (xpathObj);
+	}
+
+	/* a) Not marked => No XMP Rights information */
+	if (!marked) {
+		xmlFree (marked);
+		return NULL;
+	}
+
+	license = ev_document_license_new ();
+
+	/* b) Marked False => Public Domain, no copyrighted material and no
+	 * license needed */
+	if (g_strrstr ((char *) marked, "False") != NULL) {
+		license->text = g_strdup (_("This work is in the Public Domain"));
+	/* c) Marked True => Copyrighted material */
+	} else {
+		/* Checking usage terms as defined by the XMP Rights Management
+		 * Schema. This field is recomended to be checked by Creative
+		 * Commons */
+		/* 1) checking for a suitable localized string */
+		language_string = pango_language_to_string (gtk_get_default_language ());
+		tags = g_strsplit (language_string, "-", -1);
+		i = g_strv_length (tags);
+		while (i-- && !license->text) {
+			tag = g_strdup (tags[0]);
+			for (j = 1; j <= i; j++) {
+				tag_aux = g_strdup_printf ("%s-%s", tag, tags[j]);
+				g_free (tag);
+				tag = tag_aux;
+			}
+			aux = g_strdup_printf (LICENSE_TEXT, tag);
+			xpathObj = xmlXPathEvalExpression (BAD_CAST aux, xpathCtx);
+			if (xpathObj != NULL) {
+				if (xpathObj->nodesetval != NULL &&
+				    xpathObj->nodesetval->nodeNr != 0)
+					license->text = (gchar *)xmlNodeGetContent (xpathObj->nodesetval->nodeTab[0]);
+				xmlXPathFreeObject (xpathObj);
+			}
+			g_free (tag);
+			g_free (aux);
+		}
+		g_strfreev(tags);
+
+		/* 2) if not, use the default string */
+		if (!license->text) {
+			aux = g_strdup_printf (LICENSE_TEXT, "x-default");
+			xpathObj = xmlXPathEvalExpression (BAD_CAST aux, xpathCtx);
+			if (xpathObj != NULL) {
+				if (xpathObj->nodesetval != NULL &&
+				    xpathObj->nodesetval->nodeNr != 0)
+					license->text = (gchar *)xmlNodeGetContent (xpathObj->nodesetval->nodeTab[0]);
+				xmlXPathFreeObject (xpathObj);
+			}
+			g_free (aux);
+		}
+
+		/* Checking the license URI as defined by the Creative Commons
+		 * Schema. This field is recomended to be checked by Creative
+		 * Commons */
+		xpathObj = xmlXPathEvalExpression (BAD_CAST LICENSE_URI, xpathCtx);
+		if (xpathObj != NULL) {
+			if (xpathObj->nodesetval != NULL &&
+			    xpathObj->nodesetval->nodeNr != 0)
+				license->uri = (gchar *)xmlNodeGetContent (xpathObj->nodesetval->nodeTab[0]);
+			xmlXPathFreeObject (xpathObj);
+		}
+
+		/* Checking the web statement as defined by the XMP Rights
+		 * Management Schema. Checking it out is a sort of above-and-beyond
+		 * the basic recommendations by Creative Commons. It can be
+		 * considered as a "reinforcement" approach to add certainty. */
+		xpathObj = xmlXPathEvalExpression (BAD_CAST LICENSE_WEB_STATEMENT, xpathCtx);
+		if (xpathObj != NULL) {
+			if (xpathObj->nodesetval != NULL &&
+			    xpathObj->nodesetval->nodeNr != 0)
+				license->web_statement = (gchar *)xmlNodeGetContent (xpathObj->nodesetval->nodeTab[0]);
+			xmlXPathFreeObject (xpathObj);
+		}
+	}
+	xmlFree (marked);
+
+	if (!license->text && !license->uri && !license->web_statement) {
+		ev_document_license_free (license);
+		return NULL;
+	}
+
+	return license;
+}
+
+static void
+pdf_document_parse_metadata (const gchar    *metadata,
+			     EvDocumentInfo *info)
+{
+	xmlDocPtr          doc;
+	xmlXPathContextPtr xpathCtx;
+	gchar             *fmt;
+
+	doc = xmlParseMemory (metadata, strlen (metadata));
+	if (doc == NULL)
+		return;		/* invalid xml metadata */
+
+	xpathCtx = xmlXPathNewContext (doc);
+	if (xpathCtx == NULL) {
+		xmlFreeDoc (doc);
+		return;		/* invalid xpath context */
+	}
+
+	fmt = pdf_document_get_format_from_metadata (doc, xpathCtx);
+	if (fmt != NULL) {
+		g_free (info->format);
+		info->format = fmt;
+	}
+
+	info->license = pdf_document_get_license_from_metadata (doc, xpathCtx);
+
+	xmlXPathFreeContext (xpathCtx);
+	xmlFreeDoc (doc);
+}
+
 
 static EvDocumentInfo *
 pdf_document_get_info (EvDocument *document)
@@ -677,7 +685,6 @@ pdf_document_get_info (EvDocument *document)
 	PopplerPermissions permissions;
 	EvPage *page;
 	char *metadata;
-	char *fmt;
 
 	info = g_new0 (EvDocumentInfo, 1);
 
@@ -697,7 +704,8 @@ pdf_document_get_info (EvDocument *document)
 			    EV_DOCUMENT_INFO_LINEARIZED |
 			    EV_DOCUMENT_INFO_N_PAGES |
 			    EV_DOCUMENT_INFO_SECURITY | 
-		            EV_DOCUMENT_INFO_PAPER_SIZE;
+		            EV_DOCUMENT_INFO_PAPER_SIZE |
+			    EV_DOCUMENT_INFO_LICENSE;
 
 	g_object_get (PDF_DOCUMENT (document)->document,
 		      "title", &(info->title),
@@ -718,24 +726,16 @@ pdf_document_get_info (EvDocument *document)
 		      NULL);
 
 	if (metadata != NULL) {
-		fmt = pdf_document_get_format_from_metadata(metadata);
-		if (fmt != NULL) {
-			g_free (info->format);
-			info->format = fmt;
-		}
+		pdf_document_parse_metadata (metadata, info);
 		g_free (metadata);
 	}
 
 	info->n_pages = ev_document_get_n_pages (document);
 
 	if (info->n_pages > 0) {
-		page = ev_document_get_page (document, 0);
-		ev_document_get_page_size (document, page,
+		ev_document_get_page_size (document, 0,
 					   &(info->paper_width),
 					   &(info->paper_height));
-		g_object_unref (page);
-		
-
 		// Convert to mm.
 		info->paper_width = info->paper_width / 72.0f * 25.4f;
 		info->paper_height = info->paper_height / 72.0f * 25.4f;
@@ -831,19 +831,42 @@ pdf_document_get_info (EvDocument *document)
 }
 
 static void
-pdf_document_document_iface_init (EvDocumentIface *iface)
+pdf_document_class_init (PdfDocumentClass *klass)
 {
-	iface->save = pdf_document_save;
-	iface->load = pdf_document_load;
-	iface->get_n_pages = pdf_document_get_n_pages;
-	iface->get_page = pdf_document_get_page;
-	iface->get_page_size = pdf_document_get_page_size;
-	iface->get_page_label = pdf_document_get_page_label;
-	iface->has_attachments = pdf_document_has_attachments;
-	iface->get_attachments = pdf_document_get_attachments;
-	iface->render = pdf_document_render;
-	iface->get_info = pdf_document_get_info;
-};
+	GObjectClass    *g_object_class = G_OBJECT_CLASS (klass);
+	EvDocumentClass *ev_document_class = EV_DOCUMENT_CLASS (klass);
+
+	g_object_class->dispose = pdf_document_dispose;
+
+	ev_document_class->save = pdf_document_save;
+	ev_document_class->load = pdf_document_load;
+	ev_document_class->get_n_pages = pdf_document_get_n_pages;
+	ev_document_class->get_page = pdf_document_get_page;
+	ev_document_class->get_page_size = pdf_document_get_page_size;
+	ev_document_class->get_page_label = pdf_document_get_page_label;
+	ev_document_class->render = pdf_document_render;
+	ev_document_class->get_info = pdf_document_get_info;
+}
+
+/* EvDocumentSecurity */
+static gboolean
+pdf_document_has_document_security (EvDocumentSecurity *document_security)
+{
+	/* FIXME: do we really need to have this? */
+	return FALSE;
+}
+
+static void
+pdf_document_set_password (EvDocumentSecurity *document_security,
+			   const char         *password)
+{
+	PdfDocument *document = PDF_DOCUMENT (document_security);
+
+	if (document->password)
+		g_free (document->password);
+
+	document->password = g_strdup (password);
+}
 
 static void
 pdf_document_security_iface_init (EvDocumentSecurityIface *iface)
@@ -2488,7 +2511,7 @@ ev_annot_from_poppler_annot (PopplerAnnot *poppler_annot,
 	const gchar  *unimplemented_annot = NULL;
 
 	switch (poppler_annot_get_annot_type (poppler_annot)) {
-	        case POPPLER_ANNOT_TEXT:
+	        case POPPLER_ANNOT_TEXT: {
 			PopplerAnnotText *poppler_text;
 			EvAnnotationText *ev_annot_text;
 
@@ -2498,8 +2521,41 @@ ev_annot_from_poppler_annot (PopplerAnnot *poppler_annot,
 
 			ev_annot_text = EV_ANNOTATION_TEXT (ev_annot);
 			ev_annot_text->is_open = poppler_annot_text_get_is_open (poppler_text);
-
+		}
 			break;
+#ifdef HAVE_POPPLER_ANNOT_FILE_ATTACHMENT_GET_ATTACHMENT
+	        case POPPLER_ANNOT_FILE_ATTACHMENT: {
+			PopplerAnnotFileAttachment *poppler_annot_attachment;
+			EvAnnotationAttachment     *ev_annot_attachment;
+			PopplerAttachment          *poppler_attachment;
+			gchar                      *data = NULL;
+			gsize                       size;
+			GError                     *error = NULL;
+
+			poppler_annot_attachment = POPPLER_ANNOT_FILE_ATTACHMENT (poppler_annot);
+			poppler_attachment = poppler_annot_file_attachment_get_attachment (poppler_annot_attachment);
+
+			if (poppler_attachment &&
+			    attachment_save_to_buffer (poppler_attachment, &data, &size, &error)) {
+				EvAttachment *ev_attachment;
+
+				ev_attachment = ev_attachment_new (poppler_attachment->name,
+								   poppler_attachment->description,
+								   poppler_attachment->mtime,
+								   poppler_attachment->ctime,
+								   size, data);
+				ev_annot = ev_annotation_attachment_new (page, ev_attachment);
+				g_object_unref (ev_attachment);
+			} else if (error) {
+				g_warning ("%s", error->message);
+				g_error_free (error);
+			}
+
+			if (poppler_attachment)
+				g_object_unref (poppler_attachment);
+		}
+			break;
+#endif /* HAVE_POPPLER_ANNOT_FILE_ATTACHMENT_GET_ATTACHMENT */
 	        case POPPLER_ANNOT_LINK:
 	        case POPPLER_ANNOT_WIDGET:
 			/* Ignore link and widgets annots since they are already handled */
@@ -2653,6 +2709,123 @@ pdf_document_document_annotations_iface_init (EvDocumentAnnotationsIface *iface)
 	iface->annotation_set_contents = pdf_document_annotations_annotation_set_contents;
 }
 
+/* Attachments */
+struct SaveToBufferData {
+	gchar *buffer;
+	gsize len, max;
+};
+
+static gboolean
+attachment_save_to_buffer_callback (const gchar  *buf,
+				    gsize         count,
+				    gpointer      user_data,
+				    GError      **error)
+{
+	struct SaveToBufferData *sdata = (SaveToBufferData *)user_data;
+	gchar *new_buffer;
+	gsize new_max;
+
+	if (sdata->len + count > sdata->max) {
+		new_max = MAX (sdata->max * 2, sdata->len + count);
+		new_buffer = (gchar *)g_realloc (sdata->buffer, new_max);
+
+		sdata->buffer = new_buffer;
+		sdata->max = new_max;
+	}
+
+	memcpy (sdata->buffer + sdata->len, buf, count);
+	sdata->len += count;
+
+	return TRUE;
+}
+
+static gboolean
+attachment_save_to_buffer (PopplerAttachment  *attachment,
+			   gchar             **buffer,
+			   gsize              *buffer_size,
+			   GError            **error)
+{
+	static const gint initial_max = 1024;
+	struct SaveToBufferData sdata;
+
+	*buffer = NULL;
+	*buffer_size = 0;
+
+	sdata.buffer = (gchar *) g_malloc (initial_max);
+	sdata.max = initial_max;
+	sdata.len = 0;
+
+	if (! poppler_attachment_save_to_callback (attachment,
+						   attachment_save_to_buffer_callback,
+						   &sdata,
+						   error)) {
+		g_free (sdata.buffer);
+		return FALSE;
+	}
+
+	*buffer = sdata.buffer;
+	*buffer_size = sdata.len;
+
+	return TRUE;
+}
+
+static GList *
+pdf_document_attachments_get_attachments (EvDocumentAttachments *document)
+{
+	PdfDocument *pdf_document = PDF_DOCUMENT (document);
+	GList *attachments;
+	GList *list;
+	GList *retval = NULL;
+
+	attachments = poppler_document_get_attachments (pdf_document->document);
+
+	for (list = attachments; list; list = list->next) {
+		PopplerAttachment *attachment;
+		EvAttachment *ev_attachment;
+		gchar *data = NULL;
+		gsize size;
+		GError *error = NULL;
+
+		attachment = (PopplerAttachment *) list->data;
+
+		if (attachment_save_to_buffer (attachment, &data, &size, &error)) {
+			ev_attachment = ev_attachment_new (attachment->name,
+							   attachment->description,
+							   attachment->mtime,
+							   attachment->ctime,
+							   size, data);
+
+			retval = g_list_prepend (retval, ev_attachment);
+		} else {
+			if (error) {
+				g_warning ("%s", error->message);
+				g_error_free (error);
+
+				g_free (data);
+			}
+		}
+
+		g_object_unref (attachment);
+	}
+
+	return g_list_reverse (retval);
+}
+
+static gboolean
+pdf_document_attachments_has_attachments (EvDocumentAttachments *document)
+{
+	PdfDocument *pdf_document = PDF_DOCUMENT (document);
+
+	return poppler_document_has_attachments (pdf_document->document);
+}
+
+static void
+pdf_document_document_attachments_iface_init (EvDocumentAttachmentsIface *iface)
+{
+	iface->has_attachments = pdf_document_attachments_has_attachments;
+	iface->get_attachments = pdf_document_attachments_get_attachments;
+}
+
 /* Layers */
 static gboolean
 pdf_document_layers_has_layers (EvDocumentLayers *document)
