@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ev-document-misc.h 3381 2009-01-25 15:08:57Z chpe $
+ *  $Id$
  */
 
 #if !defined (__EV_EVINCE_DOCUMENT_H_INSIDE__) && !defined (EVINCE_COMPILATION)
@@ -51,6 +51,8 @@ cairo_surface_t *ev_document_misc_surface_rotate_and_scale (cairo_surface_t *sur
 							    gint             dest_width,
 							    gint             dest_height,
 							    gint             dest_rotation);
+void             ev_document_misc_invert_surface (cairo_surface_t *surface);
+void		 ev_document_misc_invert_pixbuf  (GdkPixbuf       *pixbuf);
 
 G_END_DECLS
 
