@@ -1,5 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; c-indent-level: 8 -*- */
 /*
+ *  Copyright (C) 2009 Carlos Garcia Campos
  *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -39,13 +40,15 @@ G_BEGIN_DECLS
 
 #define EV_TYPE_DOCUMENT            (ev_document_get_type ())
 #define EV_DOCUMENT(o)              (G_TYPE_CHECK_INSTANCE_CAST ((o), EV_TYPE_DOCUMENT, EvDocument))
-#define EV_DOCUMENT_IFACE(k)        (G_TYPE_CHECK_CLASS_CAST((k), EV_TYPE_DOCUMENT, EvDocumentIface))
+#define EV_DOCUMENT_CLASS(k)        (G_TYPE_CHECK_CLASS_CAST((k), EV_TYPE_DOCUMENT, EvDocumentClass))
 #define EV_IS_DOCUMENT(o)           (G_TYPE_CHECK_INSTANCE_TYPE ((o), EV_TYPE_DOCUMENT))
-#define EV_IS_DOCUMENT_IFACE(k)     (G_TYPE_CHECK_CLASS_TYPE ((k), EV_TYPE_DOCUMENT))
-#define EV_DOCUMENT_GET_IFACE(inst) (G_TYPE_INSTANCE_GET_INTERFACE ((inst), EV_TYPE_DOCUMENT, EvDocumentIface))
+#define EV_IS_DOCUMENT_CLASS(k)     (G_TYPE_CHECK_CLASS_TYPE ((k), EV_TYPE_DOCUMENT))
+#define EV_DOCUMENT_GET_CLASS(inst) (G_TYPE_INSTANCE_GET_CLASS ((inst), EV_TYPE_DOCUMENT, EvDocumentClass))
 
 typedef struct _EvDocument        EvDocument;
-typedef struct _EvDocumentIface   EvDocumentIface;
+typedef struct _EvDocumentClass   EvDocumentClass;
+typedef struct _EvDocumentPrivate EvDocumentPrivate;
+
 typedef struct _EvPageCache       EvPageCache;
 typedef struct _EvPageCacheClass  EvPageCacheClass;
 
@@ -64,78 +67,102 @@ typedef struct {
         double y;
 } EvPoint;
 
-typedef struct {
-        double x1;
-        double y1;
-        double x2;
-        double y2;
-} EvRectangle;
+typedef struct _EvRectangle EvRectangle;
 
-struct _EvDocumentIface
+struct _EvDocument
 {
-        GTypeInterface base_iface;
+	GObject base;
 
-        /* Methods  */
+	EvDocumentPrivate *priv;
+};
+
+struct _EvDocumentClass
+{
+        GObjectClass base_class;
+
+        /* Virtual Methods  */
         gboolean          (* load)            (EvDocument      *document,
                                                const char      *uri,
                                                GError         **error);
         gboolean          (* save)            (EvDocument      *document,
                                                const char      *uri,
                                                GError         **error);
-        int               (* get_n_pages)     (EvDocument      *document);
+        gint              (* get_n_pages)     (EvDocument      *document);
 	EvPage          * (* get_page)        (EvDocument      *document,
 					       gint             index);
         void              (* get_page_size)   (EvDocument      *document,
                                                EvPage          *page,
                                                double          *width,
                                                double          *height);
-        char            * (* get_page_label)  (EvDocument      *document,
+        gchar           * (* get_page_label)  (EvDocument      *document,
                                                EvPage          *page);
-        gboolean          (* has_attachments) (EvDocument      *document);
-        GList           * (* get_attachments) (EvDocument      *document);
         cairo_surface_t * (* render)          (EvDocument      *document,
                                                EvRenderContext *rc);
-        EvDocumentInfo *  (* get_info)        (EvDocument      *document);
+        EvDocumentInfo  * (* get_info)        (EvDocument      *document);
 };
 
-GType            ev_document_get_type         (void) G_GNUC_CONST;
-GQuark           ev_document_error_quark      (void);
+GType            ev_document_get_type             (void) G_GNUC_CONST;
+GQuark           ev_document_error_quark          (void);
 
 /* Document mutex */
-GMutex          *ev_document_get_doc_mutex    (void);
-void             ev_document_doc_mutex_lock   (void);
-void             ev_document_doc_mutex_unlock (void);
-gboolean         ev_document_doc_mutex_trylock(void);
+GMutex          *ev_document_get_doc_mutex        (void);
+void             ev_document_doc_mutex_lock       (void);
+void             ev_document_doc_mutex_unlock     (void);
+gboolean         ev_document_doc_mutex_trylock    (void);
 
 /* FontConfig mutex */
-GMutex          *ev_document_get_fc_mutex     (void);
-void             ev_document_fc_mutex_lock    (void);
-void             ev_document_fc_mutex_unlock  (void);
-gboolean         ev_document_fc_mutex_trylock (void);
+GMutex          *ev_document_get_fc_mutex         (void);
+void             ev_document_fc_mutex_lock        (void);
+void             ev_document_fc_mutex_unlock      (void);
+gboolean         ev_document_fc_mutex_trylock     (void);
 
-EvDocumentInfo  *ev_document_get_info         (EvDocument      *document);
-gboolean         ev_document_load             (EvDocument      *document,
-                                               const char      *uri,
-                                               GError         **error);
-gboolean         ev_document_save             (EvDocument      *document,
-                                               const char      *uri,
-                                               GError         **error);
-int              ev_document_get_n_pages      (EvDocument      *document);
-EvPage          *ev_document_get_page         (EvDocument      *document,
-					       gint             index);
-void             ev_document_get_page_size    (EvDocument      *document,
-                                               EvPage          *page,
-                                               double          *width,
-                                               double          *height);
-char            *ev_document_get_page_label   (EvDocument      *document,
-                                               EvPage          *page);
-gboolean         ev_document_has_attachments  (EvDocument      *document);
-GList           *ev_document_get_attachments  (EvDocument      *document);
-cairo_surface_t *ev_document_render           (EvDocument      *document,
-                                               EvRenderContext *rc);
+EvDocumentInfo  *ev_document_get_info             (EvDocument      *document);
+gboolean         ev_document_load                 (EvDocument      *document,
+						   const char      *uri,
+						   GError         **error);
+gboolean         ev_document_save                 (EvDocument      *document,
+						   const char      *uri,
+						   GError         **error);
+gint             ev_document_get_n_pages          (EvDocument      *document);
+EvPage          *ev_document_get_page             (EvDocument      *document,
+						   gint             index);
+void             ev_document_get_page_size        (EvDocument      *document,
+						   gint             page_index,
+						   double          *width,
+						   double          *height);
+gchar           *ev_document_get_page_label       (EvDocument      *document,
+						   gint             page_index);
+cairo_surface_t *ev_document_render               (EvDocument      *document,
+						   EvRenderContext *rc);
+const gchar     *ev_document_get_uri              (EvDocument      *document);
+const gchar     *ev_document_get_title            (EvDocument      *document);
+gboolean         ev_document_is_page_size_uniform (EvDocument      *document);
+void             ev_document_get_max_page_size    (EvDocument      *document,
+						   gdouble         *width,
+						   gdouble         *height);
+gboolean         ev_document_check_dimensions     (EvDocument      *document);
+gint             ev_document_get_max_label_len    (EvDocument      *document);
+gboolean         ev_document_has_text_page_labels (EvDocument      *document);
+gboolean         ev_document_find_page_by_label   (EvDocument      *document,
+						   const gchar     *page_label,
+						   gint            *page_index);
 
-gint            ev_rect_cmp                   (EvRectangle     *a,
-                                               EvRectangle     *b);
+gint             ev_rect_cmp                      (EvRectangle     *a,
+					           EvRectangle     *b);
+
+#define EV_TYPE_RECTANGLE (ev_rectangle_get_type ())
+struct _EvRectangle
+{
+	gdouble x1;
+	gdouble y1;
+	gdouble x2;
+	gdouble y2;
+};
+
+GType        ev_rectangle_get_type (void) G_GNUC_CONST;
+EvRectangle *ev_rectangle_new      (void);
+EvRectangle *ev_rectangle_copy     (EvRectangle *ev_rect);
+void         ev_rectangle_free     (EvRectangle *ev_rect);
 
 /* convenience macro to ease interface addition in the CODE
  * section of EV_BACKEND_REGISTER_WITH_CODE (this macro relies on
@@ -182,7 +209,7 @@ static void     backend_name##_class_intern_init (gpointer klass)		\
 G_MODULE_EXPORT GType								\
 register_evince_backend (GTypeModule *module)					\
 {										\
-	const GTypeInfo our_info = {  				        \
+	const GTypeInfo our_info = {  				                \
 		sizeof (BackendName##Class),					\
 		NULL, /* base_init */						\
 		NULL, /* base_finalize */					\
@@ -199,14 +226,11 @@ register_evince_backend (GTypeModule *module)					\
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");			\
                                                                                 \
 	g_define_type_id = g_type_module_register_type (module,		        \
-					    G_TYPE_OBJECT,			\
-					    #BackendName,			\
-					    &our_info,				\
-					   (GTypeFlags)0);	                \
+					                EV_TYPE_DOCUMENT,	\
+					                #BackendName,		\
+					                &our_info,		\
+					                (GTypeFlags)0);	        \
 							                        \
-	EV_BACKEND_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT,                       \
-                               backend_name##_document_iface_init);             \
-										\
 	CODE									\
 										\
 	return g_define_type_id;						\
@@ -269,6 +293,25 @@ type_name##_get_type (void)
 	}                                                                                    \
 	return g_define_type_id__volatile;                                                   \
 }
+
+/*
+ * A convenience macro for boxed type implementations, which defines a
+ * type_name_get_type() function registering the boxed type.
+ */
+#define EV_DEFINE_BOXED_TYPE(TypeName, type_name, copy_func, free_func)               \
+GType                                                                                 \
+type_name##_get_type (void)                                                           \
+{                                                                                     \
+        static volatile gsize g_define_type_id__volatile = 0;                         \
+	if (g_once_init_enter (&g_define_type_id__volatile)) {                        \
+	        GType g_define_type_id =                                              \
+		    g_boxed_type_register_static (g_intern_static_string (#TypeName), \
+		                                  (GBoxedCopyFunc) copy_func,         \
+						  (GBoxedFreeFunc) free_func);        \
+		g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);    \
+	}                                                                             \
+	return g_define_type_id__volatile;                                            \
+}
 		
 G_END_DECLS
 
